/**
 * Copyright (c) 2024 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#include "Heavy_WSTD_SMTHR.hpp"

#include <new>

#define Context(_c) static_cast<Heavy_WSTD_SMTHR *>(_c)


/*
 * C Functions
 */

extern "C" {
  HV_EXPORT HeavyContextInterface *hv_WSTD_SMTHR_new(double sampleRate) {
    // allocate aligned memory
    void *ptr = hv_malloc(sizeof(Heavy_WSTD_SMTHR));
    // ensure non-null
    if (!ptr) return nullptr;
    // call constructor
    new(ptr) Heavy_WSTD_SMTHR(sampleRate);
    return Context(ptr);
  }

  HV_EXPORT HeavyContextInterface *hv_WSTD_SMTHR_new_with_options(double sampleRate,
      int poolKb, int inQueueKb, int outQueueKb) {
    // allocate aligned memory
    void *ptr = hv_malloc(sizeof(Heavy_WSTD_SMTHR));
    // ensure non-null
    if (!ptr) return nullptr;
    // call constructor
    new(ptr) Heavy_WSTD_SMTHR(sampleRate, poolKb, inQueueKb, outQueueKb);
    return Context(ptr);
  }

  HV_EXPORT void hv_WSTD_SMTHR_free(HeavyContextInterface *instance) {
    // call destructor
    Context(instance)->~Heavy_WSTD_SMTHR();
    // free memory
    hv_free(instance);
  }
} // extern "C"







/*
 * Class Functions
 */

Heavy_WSTD_SMTHR::Heavy_WSTD_SMTHR(double sampleRate, int poolKb, int inQueueKb, int outQueueKb)
    : HeavyContext(sampleRate, poolKb, inQueueKb, outQueueKb) {
  numBytes += sLine_init(&sLine_W7Wj0P0v);
  numBytes += sEnv_init(&sEnv_rlN1kBuk, 512, 256);
  numBytes += sEnv_init(&sEnv_JrkxgvBy, 512, 256);
  numBytes += sEnv_init(&sEnv_V7lZT5tR, 512, 256);
  numBytes += sEnv_init(&sEnv_jag4T4TF, 512, 256);
  numBytes += sLine_init(&sLine_tBwgqSGc);
  numBytes += sLine_init(&sLine_wzJqRPTu);
  numBytes += cVar_init_f(&cVar_y6wqGvDL, 0.0f);
  numBytes += cVar_init_f(&cVar_jW32L8Ox, 0.0f);
  numBytes += sVarf_init(&sVarf_61sdITrH, 0.0f, 0.0f, false);
  numBytes += cVar_init_f(&cVar_padN5u6e, 0.0f);
  numBytes += cIf_init(&cIf_jBe6eqCf, false);
  numBytes += cIf_init(&cIf_NEw756f9, false);
  numBytes += cBinop_init(&cBinop_noJmy8AN, 0.0f); // __sub
  numBytes += sVarf_init(&sVarf_EUOq1J7H, 0.0f, 0.0f, false);
  numBytes += cVar_init_f(&cVar_uC2A4pdD, 0.0f);
  numBytes += cIf_init(&cIf_57X1F4CP, false);
  numBytes += cIf_init(&cIf_R696Jz9t, false);
  numBytes += cBinop_init(&cBinop_eKHsV4qx, 0.0f); // __sub
  numBytes += cVar_init_f(&cVar_kGU2XyBk, 1.0f);
  numBytes += sVarf_init(&sVarf_9dcUyjk1, 0.0f, 0.0f, false);
  numBytes += sVarf_init(&sVarf_w4kEDbBi, 0.0f, 0.0f, false);
  numBytes += sVarf_init(&sVarf_hT2aBYpw, 0.0f, 0.0f, false);
  numBytes += sVarf_init(&sVarf_aGPqpTGu, 0.0f, 0.0f, false);
  
  // schedule a message to trigger all loadbangs via the __hv_init receiver
  scheduleMessageForReceiver(0xCE5CC65B, msg_initWithBang(HV_MESSAGE_ON_STACK(1), 0));
}

Heavy_WSTD_SMTHR::~Heavy_WSTD_SMTHR() {
  sEnv_free(&sEnv_rlN1kBuk);
  sEnv_free(&sEnv_JrkxgvBy);
  sEnv_free(&sEnv_V7lZT5tR);
  sEnv_free(&sEnv_jag4T4TF);
}

HvTable *Heavy_WSTD_SMTHR::getTableForHash(hv_uint32_t tableHash) {
  return nullptr;
}

void Heavy_WSTD_SMTHR::scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) {
  switch (receiverHash) {
    case 0xCB23DC60: { // Lmtr
      mq_addMessageByTimestamp(&mq, m, 0, &cReceive_Szb8Wc4Y_sendMessage);
      break;
    }
    case 0x97D46AC6: { // Mix
      mq_addMessageByTimestamp(&mq, m, 0, &cReceive_0XL8N963_sendMessage);
      break;
    }
    case 0xD88E26F9: { // Smthr
      mq_addMessageByTimestamp(&mq, m, 0, &cReceive_2Tao769u_sendMessage);
      break;
    }
    case 0xCF8296B9: { // 1001-mix
      mq_addMessageByTimestamp(&mq, m, 0, &cReceive_9ULYhTfJ_sendMessage);
      break;
    }
    case 0xCE5CC65B: { // __hv_init
      mq_addMessageByTimestamp(&mq, m, 0, &cReceive_UXMNvBgN_sendMessage);
      break;
    }
    default: return;
  }
}

int Heavy_WSTD_SMTHR::getParameterInfo(int index, HvParameterInfo *info) {
  if (info != nullptr) {
    switch (index) {
      case 0: {
        info->name = "Lmtr";
        info->hash = 0xCB23DC60;
        info->type = HvParameterType::HV_PARAM_TYPE_PARAMETER_IN;
        info->minVal = 0.0f;
        info->maxVal = 1.0f;
        info->defaultVal = 1.0f;
        break;
      }
      case 1: {
        info->name = "Mix";
        info->hash = 0x97D46AC6;
        info->type = HvParameterType::HV_PARAM_TYPE_PARAMETER_IN;
        info->minVal = 0.0f;
        info->maxVal = 100.0f;
        info->defaultVal = 50.0f;
        break;
      }
      case 2: {
        info->name = "Smthr";
        info->hash = 0xD88E26F9;
        info->type = HvParameterType::HV_PARAM_TYPE_PARAMETER_IN;
        info->minVal = 1.0f;
        info->maxVal = 13.37f;
        info->defaultVal = 1.0f;
        break;
      }
      default: {
        info->name = "invalid parameter index";
        info->hash = 0;
        info->type = HvParameterType::HV_PARAM_TYPE_PARAMETER_IN;
        info->minVal = 0.0f;
        info->maxVal = 0.0f;
        info->defaultVal = 0.0f;
        break;
      }
    }
  }
  return 3;
}



/*
 * Send Function Implementations
 */


void Heavy_WSTD_SMTHR::cBinop_B9H7IlYo_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_MAX, 0.0f, 0, m, &cBinop_LlK7UeOm_sendMessage);
}

void Heavy_WSTD_SMTHR::cBinop_LlK7UeOm_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_MULTIPLY, 2.0f, 0, m, &cBinop_zoukrnrl_sendMessage);
}

void Heavy_WSTD_SMTHR::cBinop_zoukrnrl_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_SUBTRACT, 1.0f, 0, m, &cBinop_Ci7u7ggl_sendMessage);
}

void Heavy_WSTD_SMTHR::cBinop_Ci7u7ggl_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_cvvkPe5X_sendMessage);
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_ohyRKWKD_sendMessage);
}

void Heavy_WSTD_SMTHR::cCast_ohyRKWKD_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_PLYntzBs_sendMessage(_c, 0, m);
}

void Heavy_WSTD_SMTHR::cCast_cvvkPe5X_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_ADD, 1.0f, 0, m, &cBinop_s3CIYc5N_sendMessage);
}

void Heavy_WSTD_SMTHR::cBinop_s3CIYc5N_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_MULTIPLY, 0.5f, 0, m, &cBinop_Mh8OfyY9_sendMessage);
}

void Heavy_WSTD_SMTHR::cMsg_PLYntzBs_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(2);
  msg_init(m, 2, msg_getTimestamp(n));
  msg_setFloat(m, 0, 1.0f);
  msg_setElementToFrom(m, 1, n, 0);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_SUBTRACT, 0.0f, 0, m, &cBinop_1ZSNYhZJ_sendMessage);
}

void Heavy_WSTD_SMTHR::cBinop_1ZSNYhZJ_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_MULTIPLY, 0.5f, 0, m, &cBinop_TtvFouQp_sendMessage);
}

void Heavy_WSTD_SMTHR::cBinop_Mh8OfyY9_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_4k7bzjUT_sendMessage(_c, 0, m);
}

void Heavy_WSTD_SMTHR::cBinop_TtvFouQp_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_NOZxxaJV_sendMessage(_c, 0, m);
}

void Heavy_WSTD_SMTHR::cVar_y6wqGvDL_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cSend_6h0oHgGc_sendMessage(_c, 0, m);
}

void Heavy_WSTD_SMTHR::cCast_MYwcsbTp_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_AC0DJS1M_sendMessage(_c, 0, m);
  cMsg_nLUjUcFQ_sendMessage(_c, 0, m);
}

void Heavy_WSTD_SMTHR::cVar_jW32L8Ox_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_KqiTMlE5_sendMessage(_c, 0, m);
}

void Heavy_WSTD_SMTHR::cVar_padN5u6e_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_onMessage(_c, &Context(_c)->cBinop_noJmy8AN, HV_BINOP_SUBTRACT, 0, m, &cBinop_noJmy8AN_sendMessage);
}

void Heavy_WSTD_SMTHR::cIf_jBe6eqCf_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  switch (letIn) {
    case 0: {
      break;
    }
    case 1: {
      sVarf_onMessage(_c, &Context(_c)->sVarf_61sdITrH, m);
      break;
    }
    default: return;
  }
}

void Heavy_WSTD_SMTHR::cIf_NEw756f9_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  switch (letIn) {
    case 0: {
      break;
    }
    case 1: {
      sVarf_onMessage(_c, &Context(_c)->sVarf_61sdITrH, m);
      break;
    }
    default: return;
  }
}

void Heavy_WSTD_SMTHR::sEnv_rlN1kBuk_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_padN5u6e, 0, m, &cVar_padN5u6e_sendMessage);
}

void Heavy_WSTD_SMTHR::sEnv_JrkxgvBy_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_ifqia9uj_sendMessage);
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_mFBv7xvz_sendMessage);
  cCast_onMessage(_c, HV_CAST_BANG, 0, m, &cCast_RrmzAzWb_sendMessage);
}

void Heavy_WSTD_SMTHR::cBinop_noJmy8AN_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_EQtZ8xNS_sendMessage(_c, 0, m);
}

void Heavy_WSTD_SMTHR::cMsg_EQtZ8xNS_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(2);
  msg_init(m, 2, msg_getTimestamp(n));
  msg_setFloat(m, 0, 100.0f);
  msg_setElementToFrom(m, 1, n, 0);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_SUBTRACT, 0.0f, 0, m, &cBinop_YIobLhr1_sendMessage);
}

void Heavy_WSTD_SMTHR::cBinop_YIobLhr1_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_MIN, 100.0f, 0, m, &cBinop_PxOFW4im_sendMessage);
}

void Heavy_WSTD_SMTHR::cBinop_SegcaHnx_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_82Nll2cl_sendMessage);
  cCast_onMessage(_c, HV_CAST_BANG, 0, m, &cCast_gtwl35rs_sendMessage);
}

void Heavy_WSTD_SMTHR::cCast_gtwl35rs_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_OPt5V2YB_sendMessage(_c, 0, m);
}

void Heavy_WSTD_SMTHR::cCast_82Nll2cl_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cIf_onMessage(_c, &Context(_c)->cIf_jBe6eqCf, 1, m, &cIf_jBe6eqCf_sendMessage);
}

void Heavy_WSTD_SMTHR::cBinop_Z2V6L9Ml_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cIf_onMessage(_c, &Context(_c)->cIf_NEw756f9, 1, m, &cIf_NEw756f9_sendMessage);
}

void Heavy_WSTD_SMTHR::cMsg_OPt5V2YB_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(1);
  msg_init(m, 1, msg_getTimestamp(n));
  msg_setFloat(m, 0, 100.0f);
  cIf_onMessage(_c, &Context(_c)->cIf_jBe6eqCf, 0, m, &cIf_jBe6eqCf_sendMessage);
}

void Heavy_WSTD_SMTHR::cBinop_PxOFW4im_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cIf_onMessage(_c, &Context(_c)->cIf_NEw756f9, 0, m, &cIf_NEw756f9_sendMessage);
}

void Heavy_WSTD_SMTHR::cCast_mFBv7xvz_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_onMessage(_c, &Context(_c)->cBinop_noJmy8AN, HV_BINOP_SUBTRACT, 1, m, &cBinop_noJmy8AN_sendMessage);
}

void Heavy_WSTD_SMTHR::cCast_ifqia9uj_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
}

void Heavy_WSTD_SMTHR::cCast_RrmzAzWb_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_padN5u6e, 0, m, &cVar_padN5u6e_sendMessage);
}

void Heavy_WSTD_SMTHR::cVar_uC2A4pdD_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_onMessage(_c, &Context(_c)->cBinop_eKHsV4qx, HV_BINOP_SUBTRACT, 0, m, &cBinop_eKHsV4qx_sendMessage);
}

void Heavy_WSTD_SMTHR::cIf_57X1F4CP_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  switch (letIn) {
    case 0: {
      break;
    }
    case 1: {
      sVarf_onMessage(_c, &Context(_c)->sVarf_EUOq1J7H, m);
      break;
    }
    default: return;
  }
}

void Heavy_WSTD_SMTHR::cIf_R696Jz9t_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  switch (letIn) {
    case 0: {
      break;
    }
    case 1: {
      sVarf_onMessage(_c, &Context(_c)->sVarf_EUOq1J7H, m);
      break;
    }
    default: return;
  }
}

void Heavy_WSTD_SMTHR::sEnv_V7lZT5tR_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_uC2A4pdD, 0, m, &cVar_uC2A4pdD_sendMessage);
}

void Heavy_WSTD_SMTHR::sEnv_jag4T4TF_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_9V4oVLht_sendMessage);
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_HjOGrGVw_sendMessage);
  cCast_onMessage(_c, HV_CAST_BANG, 0, m, &cCast_sowPYD7g_sendMessage);
}

void Heavy_WSTD_SMTHR::cBinop_eKHsV4qx_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_JUlInfD6_sendMessage(_c, 0, m);
}

void Heavy_WSTD_SMTHR::cMsg_JUlInfD6_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(2);
  msg_init(m, 2, msg_getTimestamp(n));
  msg_setFloat(m, 0, 100.0f);
  msg_setElementToFrom(m, 1, n, 0);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_SUBTRACT, 0.0f, 0, m, &cBinop_QOTwCqbH_sendMessage);
}

void Heavy_WSTD_SMTHR::cBinop_QOTwCqbH_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_MIN, 100.0f, 0, m, &cBinop_e04InQIS_sendMessage);
}

void Heavy_WSTD_SMTHR::cBinop_AoUurYRD_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_x3QYgU05_sendMessage);
  cCast_onMessage(_c, HV_CAST_BANG, 0, m, &cCast_Ish8t7I6_sendMessage);
}

void Heavy_WSTD_SMTHR::cCast_x3QYgU05_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cIf_onMessage(_c, &Context(_c)->cIf_57X1F4CP, 1, m, &cIf_57X1F4CP_sendMessage);
}

void Heavy_WSTD_SMTHR::cCast_Ish8t7I6_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_l3RlEmm4_sendMessage(_c, 0, m);
}

void Heavy_WSTD_SMTHR::cBinop_AQ8Gzxgn_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cIf_onMessage(_c, &Context(_c)->cIf_R696Jz9t, 1, m, &cIf_R696Jz9t_sendMessage);
}

void Heavy_WSTD_SMTHR::cMsg_l3RlEmm4_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(1);
  msg_init(m, 1, msg_getTimestamp(n));
  msg_setFloat(m, 0, 100.0f);
  cIf_onMessage(_c, &Context(_c)->cIf_57X1F4CP, 0, m, &cIf_57X1F4CP_sendMessage);
}

void Heavy_WSTD_SMTHR::cBinop_e04InQIS_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cIf_onMessage(_c, &Context(_c)->cIf_R696Jz9t, 0, m, &cIf_R696Jz9t_sendMessage);
}

void Heavy_WSTD_SMTHR::cCast_9V4oVLht_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
}

void Heavy_WSTD_SMTHR::cCast_sowPYD7g_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_uC2A4pdD, 0, m, &cVar_uC2A4pdD_sendMessage);
}

void Heavy_WSTD_SMTHR::cCast_HjOGrGVw_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_onMessage(_c, &Context(_c)->cBinop_eKHsV4qx, HV_BINOP_SUBTRACT, 1, m, &cBinop_eKHsV4qx_sendMessage);
}

void Heavy_WSTD_SMTHR::cVar_kGU2XyBk_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_EQ, 0.0f, 0, m, &cBinop_jRl0OjLN_sendMessage);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_EQ, 0.0f, 0, m, &cBinop_SegcaHnx_sendMessage);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_EQ, 1.0f, 0, m, &cBinop_Z2V6L9Ml_sendMessage);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_EQ, 0.0f, 0, m, &cBinop_AoUurYRD_sendMessage);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_EQ, 1.0f, 0, m, &cBinop_AQ8Gzxgn_sendMessage);
}

void Heavy_WSTD_SMTHR::cBinop_jRl0OjLN_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_kGU2XyBk, 1, m, &cVar_kGU2XyBk_sendMessage);
}

void Heavy_WSTD_SMTHR::cMsg_4k7bzjUT_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(2);
  msg_init(m, 2, msg_getTimestamp(n));
  msg_setElementToFrom(m, 0, n, 0);
  msg_setFloat(m, 1, 100.0f);
  sLine_onMessage(_c, &Context(_c)->sLine_wzJqRPTu, 0, m, NULL);
}

void Heavy_WSTD_SMTHR::cMsg_NOZxxaJV_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(2);
  msg_init(m, 2, msg_getTimestamp(n));
  msg_setElementToFrom(m, 0, n, 0);
  msg_setFloat(m, 1, 100.0f);
  sLine_onMessage(_c, &Context(_c)->sLine_tBwgqSGc, 0, m, NULL);
}

void Heavy_WSTD_SMTHR::cSend_6h0oHgGc_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cReceive_9ULYhTfJ_sendMessage(_c, 0, m);
}

void Heavy_WSTD_SMTHR::cBinop_9rTDt0Vg_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_y6wqGvDL, 0, m, &cVar_y6wqGvDL_sendMessage);
}

void Heavy_WSTD_SMTHR::cMsg_nLUjUcFQ_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(1);
  msg_init(m, 1, msg_getTimestamp(n));
  msg_setFloat(m, 0, 50.0f);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_DIVIDE, 100.0f, 0, m, &cBinop_9rTDt0Vg_sendMessage);
}

void Heavy_WSTD_SMTHR::cMsg_KqiTMlE5_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(2);
  msg_init(m, 2, msg_getTimestamp(n));
  msg_setElementToFrom(m, 0, n, 0);
  msg_setFloat(m, 1, 20.0f);
  sLine_onMessage(_c, &Context(_c)->sLine_W7Wj0P0v, 0, m, NULL);
}

void Heavy_WSTD_SMTHR::cMsg_AC0DJS1M_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(1);
  msg_init(m, 1, msg_getTimestamp(n));
  msg_setFloat(m, 0, 1.0f);
  cVar_onMessage(_c, &Context(_c)->cVar_jW32L8Ox, 0, m, &cVar_jW32L8Ox_sendMessage);
  cVar_onMessage(_c, &Context(_c)->cVar_kGU2XyBk, 0, m, &cVar_kGU2XyBk_sendMessage);
}

void Heavy_WSTD_SMTHR::cReceive_9ULYhTfJ_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_MIN, 1.0f, 0, m, &cBinop_B9H7IlYo_sendMessage);
}

void Heavy_WSTD_SMTHR::cReceive_0XL8N963_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_DIVIDE, 100.0f, 0, m, &cBinop_9rTDt0Vg_sendMessage);
}

void Heavy_WSTD_SMTHR::cReceive_UXMNvBgN_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cCast_onMessage(_c, HV_CAST_BANG, 0, m, &cCast_MYwcsbTp_sendMessage);
}

void Heavy_WSTD_SMTHR::cReceive_Szb8Wc4Y_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_kGU2XyBk, 0, m, &cVar_kGU2XyBk_sendMessage);
}

void Heavy_WSTD_SMTHR::cReceive_2Tao769u_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_jW32L8Ox, 0, m, &cVar_jW32L8Ox_sendMessage);
}




/*
 * Context Process Implementation
 */

int Heavy_WSTD_SMTHR::process(float **inputBuffers, float **outputBuffers, int n) {
  while (hLp_hasData(&inQueue)) {
    hv_uint32_t numBytes = 0;
    ReceiverMessagePair *p = reinterpret_cast<ReceiverMessagePair *>(hLp_getReadBuffer(&inQueue, &numBytes));
    hv_assert(numBytes >= sizeof(ReceiverMessagePair));
    scheduleMessageForReceiver(p->receiverHash, &p->msg);
    hLp_consume(&inQueue);
  }

  sendBangToReceiver(0xDD21C0EB); // send to __hv_bang~ on next cycle
  const int n4 = n & ~HV_N_SIMD_MASK; // ensure that the block size is a multiple of HV_N_SIMD

  // temporary signal vars
  hv_bufferf_t Bf0, Bf1, Bf2, Bf3, Bf4, Bf5;

  // input and output vars
  hv_bufferf_t O0, O1;
  hv_bufferf_t I0, I1;

  // declare and init the zero buffer
  hv_bufferf_t ZERO; __hv_zero_f(VOf(ZERO));

  hv_uint32_t nextBlock = blockStartTimestamp;
  for (int n = 0; n < n4; n += HV_N_SIMD) {

    // process all of the messages for this block
    nextBlock += HV_N_SIMD;
    while (mq_hasMessageBefore(&mq, nextBlock)) {
      MessageNode *const node = mq_peek(&mq);
      node->sendMessage(this, node->let, node->m);
      mq_pop(&mq);
    }

    // load input buffers
    __hv_load_f(inputBuffers[0]+n, VOf(I0));
    __hv_load_f(inputBuffers[1]+n, VOf(I1));

    // zero output buffers
    __hv_zero_f(VOf(O0));
    __hv_zero_f(VOf(O1));

    // process all signal functions
    __hv_varwrite_f(&sVarf_9dcUyjk1, VIf(I0));
    __hv_line_f(&sLine_W7Wj0P0v, VOf(Bf0));
    __hv_mul_f(VIf(I0), VIf(Bf0), VOf(Bf1));
    __hv_var_k_f(VOf(Bf2), 3.0f, 3.0f, 3.0f, 3.0f, 3.0f, 3.0f, 3.0f, 3.0f);
    __hv_min_f(VIf(Bf1), VIf(Bf2), VOf(Bf2));
    __hv_var_k_f(VOf(Bf1), -3.0f, -3.0f, -3.0f, -3.0f, -3.0f, -3.0f, -3.0f, -3.0f);
    __hv_max_f(VIf(Bf2), VIf(Bf1), VOf(Bf1));
    __hv_mul_f(VIf(Bf1), VIf(Bf1), VOf(Bf2));
    __hv_var_k_f(VOf(Bf3), 27.0f, 27.0f, 27.0f, 27.0f, 27.0f, 27.0f, 27.0f, 27.0f);
    __hv_add_f(VIf(Bf2), VIf(Bf3), VOf(Bf4));
    __hv_var_k_f(VOf(Bf5), 9.0f, 9.0f, 9.0f, 9.0f, 9.0f, 9.0f, 9.0f, 9.0f);
    __hv_fma_f(VIf(Bf2), VIf(Bf5), VIf(Bf3), VOf(Bf3));
    __hv_div_f(VIf(Bf4), VIf(Bf3), VOf(Bf3));
    __hv_mul_f(VIf(Bf1), VIf(Bf3), VOf(Bf3));
    sEnv_process(this, &sEnv_rlN1kBuk, VIf(Bf3), &sEnv_rlN1kBuk_sendMessage);
    sEnv_process(this, &sEnv_JrkxgvBy, VIf(I0), &sEnv_JrkxgvBy_sendMessage);
    __hv_varread_f(&sVarf_61sdITrH, VOf(Bf1));
    __hv_var_k_f(VOf(Bf4), 100.0f, 100.0f, 100.0f, 100.0f, 100.0f, 100.0f, 100.0f, 100.0f);
    __hv_sub_f(VIf(Bf1), VIf(Bf4), VOf(Bf4));
    __hv_var_k_f(VOf(Bf1), 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f);
    __hv_mul_f(VIf(Bf4), VIf(Bf1), VOf(Bf1));
    __hv_exp_f(VIf(Bf1), VOf(Bf1));
    __hv_mul_f(VIf(Bf3), VIf(Bf1), VOf(Bf1));
    __hv_varwrite_f(&sVarf_w4kEDbBi, VIf(Bf1));
    __hv_varwrite_f(&sVarf_hT2aBYpw, VIf(I1));
    __hv_mul_f(VIf(I1), VIf(Bf0), VOf(Bf0));
    __hv_var_k_f(VOf(Bf1), 3.0f, 3.0f, 3.0f, 3.0f, 3.0f, 3.0f, 3.0f, 3.0f);
    __hv_min_f(VIf(Bf0), VIf(Bf1), VOf(Bf1));
    __hv_var_k_f(VOf(Bf0), -3.0f, -3.0f, -3.0f, -3.0f, -3.0f, -3.0f, -3.0f, -3.0f);
    __hv_max_f(VIf(Bf1), VIf(Bf0), VOf(Bf0));
    __hv_mul_f(VIf(Bf0), VIf(Bf0), VOf(Bf1));
    __hv_var_k_f(VOf(Bf3), 27.0f, 27.0f, 27.0f, 27.0f, 27.0f, 27.0f, 27.0f, 27.0f);
    __hv_add_f(VIf(Bf1), VIf(Bf3), VOf(Bf4));
    __hv_var_k_f(VOf(Bf5), 9.0f, 9.0f, 9.0f, 9.0f, 9.0f, 9.0f, 9.0f, 9.0f);
    __hv_fma_f(VIf(Bf1), VIf(Bf5), VIf(Bf3), VOf(Bf3));
    __hv_div_f(VIf(Bf4), VIf(Bf3), VOf(Bf3));
    __hv_mul_f(VIf(Bf0), VIf(Bf3), VOf(Bf3));
    sEnv_process(this, &sEnv_V7lZT5tR, VIf(Bf3), &sEnv_V7lZT5tR_sendMessage);
    sEnv_process(this, &sEnv_jag4T4TF, VIf(I1), &sEnv_jag4T4TF_sendMessage);
    __hv_varread_f(&sVarf_EUOq1J7H, VOf(Bf0));
    __hv_var_k_f(VOf(Bf4), 100.0f, 100.0f, 100.0f, 100.0f, 100.0f, 100.0f, 100.0f, 100.0f);
    __hv_sub_f(VIf(Bf0), VIf(Bf4), VOf(Bf4));
    __hv_var_k_f(VOf(Bf0), 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f);
    __hv_mul_f(VIf(Bf4), VIf(Bf0), VOf(Bf0));
    __hv_exp_f(VIf(Bf0), VOf(Bf0));
    __hv_mul_f(VIf(Bf3), VIf(Bf0), VOf(Bf0));
    __hv_varwrite_f(&sVarf_aGPqpTGu, VIf(Bf0));
    __hv_varread_f(&sVarf_9dcUyjk1, VOf(Bf0));
    __hv_line_f(&sLine_tBwgqSGc, VOf(Bf3));
    __hv_varread_f(&sVarf_w4kEDbBi, VOf(Bf4));
    __hv_line_f(&sLine_wzJqRPTu, VOf(Bf5));
    __hv_mul_f(VIf(Bf4), VIf(Bf5), VOf(Bf4));
    __hv_fma_f(VIf(Bf0), VIf(Bf3), VIf(Bf4), VOf(Bf4));
    __hv_add_f(VIf(Bf4), VIf(O0), VOf(O0));
    __hv_varread_f(&sVarf_hT2aBYpw, VOf(Bf4));
    __hv_varread_f(&sVarf_aGPqpTGu, VOf(Bf0));
    __hv_mul_f(VIf(Bf0), VIf(Bf5), VOf(Bf5));
    __hv_fma_f(VIf(Bf4), VIf(Bf3), VIf(Bf5), VOf(Bf5));
    __hv_add_f(VIf(Bf5), VIf(O1), VOf(O1));

    // save output vars to output buffer
    __hv_store_f(outputBuffers[0]+n, VIf(O0));
    __hv_store_f(outputBuffers[1]+n, VIf(O1));
  }

  blockStartTimestamp = nextBlock;

  return n4; // return the number of frames processed

}

int Heavy_WSTD_SMTHR::processInline(float *inputBuffers, float *outputBuffers, int n4) {
  hv_assert(!(n4 & HV_N_SIMD_MASK)); // ensure that n4 is a multiple of HV_N_SIMD

  // define the heavy input buffer for 2 channel(s)
  float **const bIn = reinterpret_cast<float **>(hv_alloca(2*sizeof(float *)));
  bIn[0] = inputBuffers+(0*n4);
  bIn[1] = inputBuffers+(1*n4);

  // define the heavy output buffer for 2 channel(s)
  float **const bOut = reinterpret_cast<float **>(hv_alloca(2*sizeof(float *)));
  bOut[0] = outputBuffers+(0*n4);
  bOut[1] = outputBuffers+(1*n4);

  int n = process(bIn, bOut, n4);
  return n;
}

int Heavy_WSTD_SMTHR::processInlineInterleaved(float *inputBuffers, float *outputBuffers, int n4) {
  hv_assert(n4 & ~HV_N_SIMD_MASK); // ensure that n4 is a multiple of HV_N_SIMD

  // define the heavy input buffer for 2 channel(s), uninterleave
  float *const bIn = reinterpret_cast<float *>(hv_alloca(2*n4*sizeof(float)));
  #if HV_SIMD_SSE || HV_SIMD_AVX
  for (int i = 0, j = 0; j < n4; j += 4, i += 8) {
    __m128 a = _mm_load_ps(inputBuffers+i);                // LRLR
    __m128 b = _mm_load_ps(inputBuffers+4+i);              // LRLR
    __m128 x = _mm_shuffle_ps(a, b, _MM_SHUFFLE(2,0,2,0)); // LLLL
    __m128 y = _mm_shuffle_ps(a, b, _MM_SHUFFLE(3,1,3,1)); // RRRR
    _mm_store_ps(bIn+j, x);
    _mm_store_ps(bIn+n4+j, y);
  }
  #elif HV_SIMD_NEON
  for (int i = 0, j = 0; j < n4; j += 4, i += 8) {
    float32x4x2_t a = vld2q_f32(inputBuffers+i); // load and uninterleave
    vst1q_f32(bIn+j, a.val[0]);
    vst1q_f32(bIn+n4+j, a.val[1]);
  }
  #else // HV_SIMD_NONE
  for (int j = 0; j < n4; ++j) {
    bIn[0*n4+j] = inputBuffers[0+2*j];
    bIn[1*n4+j] = inputBuffers[1+2*j];
  }
  #endif

  // define the heavy output buffer for 2 channel(s)
  float *const bOut = reinterpret_cast<float *>(hv_alloca(2*n4*sizeof(float)));

  int n = processInline(bIn, bOut, n4);

  // interleave the heavy output into the output buffer
  #if HV_SIMD_AVX
  for (int i = 0, j = 0; j < n4; j += 8, i += 16) {
    __m256 x = _mm256_load_ps(bOut+j);    // LLLLLLLL
    __m256 y = _mm256_load_ps(bOut+n4+j); // RRRRRRRR
    __m256 a = _mm256_unpacklo_ps(x, y);  // LRLRLRLR
    __m256 b = _mm256_unpackhi_ps(x, y);  // LRLRLRLR
    _mm256_store_ps(outputBuffers+i, a);
    _mm256_store_ps(outputBuffers+8+i, b);
  }
  #elif HV_SIMD_SSE
  for (int i = 0, j = 0; j < n4; j += 4, i += 8) {
    __m128 x = _mm_load_ps(bOut+j);    // LLLL
    __m128 y = _mm_load_ps(bOut+n4+j); // RRRR
    __m128 a = _mm_unpacklo_ps(x, y);  // LRLR
    __m128 b = _mm_unpackhi_ps(x, y);  // LRLR
    _mm_store_ps(outputBuffers+i, a);
    _mm_store_ps(outputBuffers+4+i, b);
  }
  #elif HV_SIMD_NEON
  // https://community.arm.com/groups/processors/blog/2012/03/13/coding-for-neon--part-5-rearranging-vectors
  for (int i = 0, j = 0; j < n4; j += 4, i += 8) {
    float32x4_t x = vld1q_f32(bOut+j);
    float32x4_t y = vld1q_f32(bOut+n4+j);
    float32x4x2_t z = {x, y};
    vst2q_f32(outputBuffers+i, z); // interleave and store
  }
  #else // HV_SIMD_NONE
  for (int i = 0; i < 2; ++i) {
    for (int j = 0; j < n4; ++j) {
      outputBuffers[i+2*j] = bOut[i*n4+j];
    }
  }
  #endif

  return n;
}
