/**
 * Copyright (c) 2024 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef _HEAVY_CONTEXT_WSTD_SMTHR_HPP_
#define _HEAVY_CONTEXT_WSTD_SMTHR_HPP_

// object includes
#include "HeavyContext.hpp"
#include "HvSignalEnvelope.h"
#include "HvSignalVar.h"
#include "HvControlCast.h"
#include "HvControlVar.h"
#include "HvSignalLine.h"
#include "HvControlBinop.h"
#include "HvMath.h"
#include "HvControlIf.h"

class Heavy_WSTD_SMTHR : public HeavyContext {

 public:
  Heavy_WSTD_SMTHR(double sampleRate, int poolKb=10, int inQueueKb=2, int outQueueKb=0);
  ~Heavy_WSTD_SMTHR();

  const char *getName() override { return "WSTD_SMTHR"; }
  int getNumInputChannels() override { return 2; }
  int getNumOutputChannels() override { return 2; }

  int process(float **inputBuffers, float **outputBuffer, int n) override;
  int processInline(float *inputBuffers, float *outputBuffer, int n) override;
  int processInlineInterleaved(float *inputBuffers, float *outputBuffer, int n) override;

  int getParameterInfo(int index, HvParameterInfo *info) override;
  struct Parameter {
    struct In {
      enum ParameterIn : hv_uint32_t {
        LMTR = 0xCB23DC60, // Lmtr
        MIX = 0x97D46AC6, // Mix
        SMTHR = 0xD88E26F9, // Smthr
      };
    };
  };

 private:
  HvTable *getTableForHash(hv_uint32_t tableHash) override;
  void scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) override;

  // static sendMessage functions
  static void cBinop_B9H7IlYo_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_LlK7UeOm_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_zoukrnrl_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Ci7u7ggl_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_ohyRKWKD_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_cvvkPe5X_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_s3CIYc5N_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_PLYntzBs_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_1ZSNYhZJ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Mh8OfyY9_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_TtvFouQp_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_y6wqGvDL_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_MYwcsbTp_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_jW32L8Ox_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_padN5u6e_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_jBe6eqCf_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_NEw756f9_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_rlN1kBuk_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_JrkxgvBy_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_noJmy8AN_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_EQtZ8xNS_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_YIobLhr1_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_SegcaHnx_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_gtwl35rs_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_82Nll2cl_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Z2V6L9Ml_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_OPt5V2YB_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_PxOFW4im_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_mFBv7xvz_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_ifqia9uj_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_RrmzAzWb_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_uC2A4pdD_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_57X1F4CP_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_R696Jz9t_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_V7lZT5tR_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_jag4T4TF_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_eKHsV4qx_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_JUlInfD6_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_QOTwCqbH_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_AoUurYRD_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_x3QYgU05_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_Ish8t7I6_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_AQ8Gzxgn_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_l3RlEmm4_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_e04InQIS_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_9V4oVLht_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_sowPYD7g_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_HjOGrGVw_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_kGU2XyBk_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_jRl0OjLN_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_4k7bzjUT_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_NOZxxaJV_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cSend_6h0oHgGc_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_9rTDt0Vg_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_nLUjUcFQ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_KqiTMlE5_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_AC0DJS1M_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_9ULYhTfJ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_0XL8N963_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_UXMNvBgN_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_Szb8Wc4Y_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_2Tao769u_sendMessage(HeavyContextInterface *, int, const HvMessage *);

  // objects
  SignalLine sLine_W7Wj0P0v;
  SignalEnvelope sEnv_rlN1kBuk;
  SignalEnvelope sEnv_JrkxgvBy;
  SignalEnvelope sEnv_V7lZT5tR;
  SignalEnvelope sEnv_jag4T4TF;
  SignalLine sLine_tBwgqSGc;
  SignalLine sLine_wzJqRPTu;
  ControlBinop cBinop_B9H7IlYo;
  ControlBinop cBinop_LlK7UeOm;
  ControlBinop cBinop_zoukrnrl;
  ControlBinop cBinop_Ci7u7ggl;
  ControlBinop cBinop_s3CIYc5N;
  ControlBinop cBinop_1ZSNYhZJ;
  ControlBinop cBinop_Mh8OfyY9;
  ControlBinop cBinop_TtvFouQp;
  ControlVar cVar_y6wqGvDL;
  ControlVar cVar_jW32L8Ox;
  SignalVarf sVarf_61sdITrH;
  ControlVar cVar_padN5u6e;
  ControlIf cIf_jBe6eqCf;
  ControlIf cIf_NEw756f9;
  ControlBinop cBinop_noJmy8AN;
  ControlBinop cBinop_YIobLhr1;
  ControlBinop cBinop_SegcaHnx;
  ControlBinop cBinop_Z2V6L9Ml;
  ControlBinop cBinop_PxOFW4im;
  SignalVarf sVarf_EUOq1J7H;
  ControlVar cVar_uC2A4pdD;
  ControlIf cIf_57X1F4CP;
  ControlIf cIf_R696Jz9t;
  ControlBinop cBinop_eKHsV4qx;
  ControlBinop cBinop_QOTwCqbH;
  ControlBinop cBinop_AoUurYRD;
  ControlBinop cBinop_AQ8Gzxgn;
  ControlBinop cBinop_e04InQIS;
  ControlVar cVar_kGU2XyBk;
  ControlBinop cBinop_jRl0OjLN;
  SignalVarf sVarf_9dcUyjk1;
  SignalVarf sVarf_w4kEDbBi;
  SignalVarf sVarf_hT2aBYpw;
  SignalVarf sVarf_aGPqpTGu;
  ControlBinop cBinop_9rTDt0Vg;
};

#endif // _HEAVY_CONTEXT_WSTD_SMTHR_HPP_
